@@ -1,4 +1,5 @@
 // Copyright (c) 2023 Sophie Herold
+// Copyright (c) 2023 FineFindus
 //
 // This program is free software: you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -15,16 +16,16 @@
 //
 // SPDX-License-Identifier: GPL-3.0-or-later
 
-///! Tiled renderer
-///!
-///! It is not always feasible or desiareble to store the complete decoded
-///! image in the VRAM. [`TiledImage`] allows to compose the parts of
-///! the image currently viewed, out of smaller [`Tiles`].
-///!
-///! This is especially important for SVGs where every the image has to
-///! be re-genearted for each zoom level. It can also be used to allow
-///! showing large JPEGs etc where the complete decoded image would not
-///! even fit in the VRAM.
+//! Tiled renderer
+//!
+//! It is not always feasible or desiareble to store the complete decoded
+//! image in the VRAM. [`TiledImage`] allows to compose the parts of
+//! the image currently viewed, out of smaller [`Tiles`].
+//!
+//! This is especially important for SVGs where every the image has to
+//! be re-genearted for each zoom level. It can also be used to allow
+//! showing large JPEGs etc where the complete decoded image would not
+//! even fit in the VRAM.
 use super::{DecoderUpdate, ImageDimensionDetails, UpdateSender};
 use crate::deps::*;
 
@@ -304,7 +305,9 @@ impl TiledImage {
     }
 
     pub fn contains(&self, zoom: f64, coordinates: Coordinates) -> bool {
-        let Some(tile_layer) = self.tile_layers.get(&zoom_to_level(zoom)) else { return false };
+        let Some(tile_layer) = self.tile_layers.get(&zoom_to_level(zoom)) else {
+            return false;
+        };
 
         tile_layer.tiles.contains_key(&coordinates)
     }
@@ -483,7 +486,9 @@ impl Tiling {
         for x in (x0 as i32..=x1 as i32).step_by(self.tile_width.into()) {
             for y in (y0 as i32..=y1 as i32).step_by(self.tile_height.into()) {
                 let area = graphene::Rect::new(x as f32, y as f32, tile_width, tile_height);
-                let Some(restricted_area) = area.restrict(original_width, original_height) else { continue; };
+                let Some(restricted_area) = area.restrict(original_width, original_height) else {
+                    continue;
+                };
 
                 let tile = TileInstructions {
                     tiling: *self,
@@ -653,15 +658,18 @@ impl FrameBufferExt for ArcSwap<FrameBuffer> {
     }
 
     fn frame_timeout(&self, elapsed: Duration) -> bool {
-        if let Some(next_frame) = self.load().images.get(1) {
-            if elapsed >= next_frame.delay {
-                self.rcu(|tiling_store| {
-                    let mut new_store = (**tiling_store).clone();
-                    new_store.images.pop_front();
-                    Arc::new(new_store)
-                });
-                return true;
-            }
+        if self
+            .load()
+            .images
+            .front()
+            .is_some_and(|next_frame| elapsed >= next_frame.delay)
+        {
+            self.rcu(|tiling_store| {
+                let mut new_store = (**tiling_store).clone();
+                new_store.images.pop_front();
+                Arc::new(new_store)
+            });
+            return true;
         }
 
         false
