@@ -2,6 +2,8 @@
 // Copyright (c) 2022-2023 Sophie Herold
 // Copyright (c) 2022 Elton A Rodrigues
 // Copyright (c) 2022 Maximiliano Sandoval R
+// Copyright (c) 2023 FineFindus
+// Copyright (c) 2023 Huan Nguyen
 // Copyright (c) 2023 Philipp Kiemle
 //
 // This program is free software: you can redistribute it and/or modify
@@ -37,11 +39,11 @@ use crate::util::spawn;
 use adw::prelude::*;
 use adw::subclass::prelude::*;
 use anyhow::Context;
-use ashpd::desktop::wallpaper;
-use ashpd::desktop::ResponseError;
+use ashpd::desktop::{wallpaper, ResponseError};
 use ashpd::Error;
 use ashpd::WindowIdentifier;
 use glib::clone;
+use glib::translate::IntoGlib;
 use gtk::CompositeTemplate;
 use once_cell::sync::Lazy;
 
@@ -52,7 +54,7 @@ use std::cell::{Cell, RefCell};
 const BUFFER: usize = 2;
 
 mod imp {
-    use once_cell::sync::OnceCell;
+    use std::cell::OnceCell;
 
     use super::*;
 
@@ -76,6 +78,8 @@ mod imp {
         #[template_child]
         pub sliding_view: TemplateChild<LpSlidingView>,
 
+        pub drag_source: gtk::DragSource,
+
         pub(super) model: RefCell<LpFileModel>,
         pub(super) preserve_content: Cell<bool>,
 
@@ -135,7 +139,7 @@ mod imp {
             // Manually mange widget layout, see `WidgetImpl` for details
             obj.set_layout_manager(None::<gtk::LayoutManager>);
 
-            let signal_group = glib::SignalGroup::new(LpImage::static_type());
+            let signal_group = glib::SignalGroup::new::<LpImage>();
             obj.connect_notify_local(
                 Some("current-page"),
                 clone!(@weak signal_group => move |obj, _| {
@@ -162,17 +166,19 @@ mod imp {
 
             self.current_image_signals.set(signal_group).unwrap();
 
-            let source = gtk::DragSource::new();
-            source.set_exclusive(true);
+            self.drag_source.set_exclusive(true);
 
-            source.connect_prepare(
+            self.drag_source.connect_prepare(
                 glib::clone!(@weak obj => @default-return None, move |gesture, _, _| {
                     let is_scrollable = obj
                         .current_page()
                         .map(|p| p.image().is_hscrollable() || p.image().is_vscrollable());
 
-                    // do scrolling if scrollable
-                    if is_scrollable == Some(true) {
+                    if is_scrollable == Some(true)
+                        || gesture.device().map(|x| x.source())
+                            == Some(gdk::InputSource::Touchscreen)
+                    {
+                        // Do scrolling if scrollable and no drag and drop on touchscreen
                         gesture.set_state(gtk::EventSequenceState::Denied);
                         None
                     } else {
@@ -182,7 +188,7 @@ mod imp {
                 }),
             );
 
-            source.connect_drag_begin(glib::clone!(@weak obj => move |source, _| {
+            self.drag_source.connect_drag_begin(glib::clone!(@weak obj => move |source, _| {
                 if let Some(paintable) = obj.current_image().and_then(|p| p.thumbnail()) {
                     // -6 for cursor width, +16 for margin in .drag-icon
                     source.set_icon(Some(&paintable), paintable.intrinsic_width() / 2 - 6 + 16, -12);
@@ -196,7 +202,7 @@ mod imp {
                 };
             }));
 
-            obj.add_controller(source);
+            obj.add_controller(self.drag_source.clone());
         }
     }
 
@@ -241,20 +247,19 @@ glib::wrapper! {
 
 #[gtk::template_callbacks]
 impl LpImageView {
-    pub fn set_image_from_file(&self, file: gio::File) {
+    pub fn set_images_from_files(&self, files: Vec<gio::File>) {
         // Add image to recently used file. Does not work in Flatpaks:
         // <https://github.com/flatpak/xdg-desktop-portal/issues/215>
-        gtk::RecentManager::default().add_item(&file.uri());
-
-        self.load_file(file);
-    }
-
-    pub fn set_images_from_files(&self, files: Vec<gio::File>) {
         let recent_manager = gtk::RecentManager::default();
         for file in files.iter() {
             recent_manager.add_item(&file.uri());
         }
-        self.load_files(files);
+
+        if files.len() == 1 {
+            self.load_file(files[0].clone());
+        } else {
+            self.load_files(files);
+        }
     }
 
     fn load_files(&self, files: Vec<gio::File>) {
@@ -304,7 +309,7 @@ impl LpImageView {
             spawn(glib::clone!(@weak self as obj, @strong file => async move {
                 if let Err(err) = obj.model().load_directory(directory.clone()).await {
                     log::warn!("Failed to load directory: {err}");
-                    obj.activate_action("win.show-toast", Some(&(err.to_string(), 1).to_variant()))
+                    obj.activate_action("win.show-toast", Some(&(err.to_string(), adw::ToastPriority::High.into_glib()).to_variant()))
                         .unwrap();
                     return;
                 }
@@ -344,10 +349,11 @@ impl LpImageView {
     fn navigate_to_file(&self, new_file: &gio::File) {
         let sliding_view = self.sliding_view();
 
-        if let Some(current_file) = self.current_file() {
-            if new_file.equal(&current_file) {
-                return;
-            }
+        if self
+            .current_file()
+            .is_some_and(|current_file| current_file.equal(new_file))
+        {
+            return;
         }
 
         let Some(new_index) = self.model().index_of(new_file) else {
@@ -414,7 +420,10 @@ impl LpImageView {
 
     fn scroll_sliding_view(&self, file: &gio::File) {
         let Some(current_page) = self.sliding_view().pages().remove(&file.uri()) else {
-            log::error!("Current path not available in sliding_view for scrolling: {}", file.uri());
+            log::error!(
+                "Current path not available in sliding_view for scrolling: {}",
+                file.uri()
+            );
             return;
         };
 
@@ -463,6 +472,9 @@ impl LpImageView {
                     if let Some(file) = image.file() {
                         log::debug!("Removing image with unsupported format {:?}", file.uri());
                         obj.model().remove(&file);
+                        if let Some(current_file) = obj.current_file() {
+                            obj.update_sliding_view(&current_file);
+                        }
                     }
                 }
             }),
@@ -480,7 +492,9 @@ impl LpImageView {
 
     /// Handle files are added or removed from directory
     fn model_content_changed_cb(&self) {
-        let Some(current_file) = self.current_file() else { return; };
+        let Some(current_file) = self.current_file() else {
+            return;
+        };
 
         // LpImage did not get the update yet
         // Update will be handled by current_image_path_changed
@@ -538,6 +552,10 @@ impl LpImageView {
             .and_then(|x| x.image().file())
     }
 
+    pub fn drag_source(&self) -> gtk::DragSource {
+        self.imp().drag_source.clone()
+    }
+
     /// Returns `true` if there is an image before the current one
     pub fn is_previous_available(&self) -> bool {
         if let Some(file) = self.current_file() {
@@ -640,10 +658,8 @@ impl LpImageView {
             .identifier(id)
             .build_uri(&uri)
             .await
+            .and_then(|req| req.response())
         {
-            // We use `1` here because we can't pass enums directly as GVariants,
-            // so we need to use the C int value of the enum.
-            // `TOAST_PRIORITY_NORMAL = 0`, and `TOAST_PRIORITY_HIGH = 1`
             Ok(_) => self
                 .activate_action(
                     "win.show-toast",
@@ -651,7 +667,7 @@ impl LpImageView {
                         &(
                             // Translators: This is a toast notification, informing the user that an image has been set as background.
                             gettext("Set as background."),
-                            1,
+                            adw::ToastPriority::High.into_glib(),
                         )
                             .to_variant(),
                     ),
@@ -661,7 +677,13 @@ impl LpImageView {
                 if !matches!(err, Error::Response(ResponseError::Cancelled)) {
                     self.activate_action(
                         "win.show-toast",
-                        Some(&(gettext("Could not set background."), 1).to_variant()),
+                        Some(
+                            &(
+                                gettext("Could not set background."),
+                                adw::ToastPriority::High.into_glib(),
+                            )
+                                .to_variant(),
+                        ),
                     )
                     .unwrap();
                 }
@@ -681,7 +703,7 @@ impl LpImageView {
             .downcast_ref::<gtk::Window>()
             .context("Could not downcast to GtkWindow")?;
 
-        LpPrint::new(image, window.clone(), None, None).run();
+        LpPrint::new(image, window.clone(), None, None).run()?;
 
         Ok(())
     }
